@@ -519,6 +519,7 @@ class ABI_EXPORT GR_Graphics
 #define COUNT_3D_COLORS 5
 
 	virtual void      setColor3D(GR_Color3D c) = 0;
+	virtual bool      getColor3D(GR_Color3D name, UT_RGBColor &color) { return false; }
 
 	//
 	// Methods to deal with background repainting as used in the Unix FE. These
@@ -751,6 +752,8 @@ class ABI_EXPORT GR_Graphics
 
 	const UT_Rect *  m_pRect;
 
+	bool m_bHave3DColors;
+
  private:
 	GR_Caret *		 m_pCaret;
     bool             _PtInPolygon(UT_Point * pts,UT_uint32 nPoints,UT_sint32 x,UT_sint32 y);
