@@ -48,11 +48,12 @@
 #include "px_CR_Strux.h"
 #include "px_CR_StruxChange.h"
 #include "px_CR_Glob.h"
-#include "fv_View.h"
 #include "fp_Run.h"
 #include "ut_debugmsg.h"
 #include "ut_assert.h"
 #include "ut_units.h"
+#include "fp_TableContainer.h"
+#include "fv_View.h"
 
 fl_EmbedLayout::fl_EmbedLayout(FL_DocLayout* pLayout, fl_DocSectionLayout* pDocSL, PL_StruxDocHandle sdh, PT_AttrPropIndex indexAP, fl_ContainerLayout * pMyContainerLayout, SectionType iSecType,fl_ContainerType myType,PTStruxType myStruxType)
  	: fl_SectionLayout(pLayout, sdh, indexAP, iSecType,myType,myStruxType,pMyContainerLayout),
@@ -60,7 +61,8 @@ fl_EmbedLayout::fl_EmbedLayout(FL_DocLayout* pLayout, fl_DocSectionLayout* pDocS
 	  m_bNeedsFormat(true),
 	  m_bIsOnPage(false),
 	  m_pDocSL(pDocSL),
-	  m_bHasEndFootnote(false)
+	  m_bHasEndFootnote(false),
+	  m_iOldSize(0)
 {
 	UT_ASSERT(m_pDocSL->getContainerType() == FL_CONTAINER_DOCSECTION);
 }
@@ -77,6 +79,7 @@ fl_EmbedLayout::~fl_EmbedLayout()
 PT_DocPosition fl_EmbedLayout::getDocPosition(void) 
 {
 	PL_StruxDocHandle sdh = getStruxDocHandle();
+	UT_return_val_if_fail( m_pLayout, 0 );
     return 	m_pLayout->getDocument()->getStruxPosition(sdh);
 }
 
@@ -87,6 +90,7 @@ PT_DocPosition fl_EmbedLayout::getDocPosition(void)
  */
 UT_uint32 fl_EmbedLayout::getLength(void)
 {
+	UT_return_val_if_fail( m_pLayout, 0 );
 	PT_DocPosition startPos = getDocPosition();
 	PL_StruxDocHandle sdhEnd = NULL;
 	PL_StruxDocHandle sdhStart = getStruxDocHandle();
@@ -211,6 +215,7 @@ void fl_EmbedLayout::updateLayout(void)
 	{
 		format();
 	}
+	m_vecFormatLayout.clear();
 	fl_ContainerLayout*	pBL = getFirstLayout();
 	while (pBL)
 	{
@@ -237,6 +242,22 @@ void fl_EmbedLayout::redrawUpdate(void)
 	}
 }
 
+void fl_EmbedLayout::setNeedsReformat(fl_ContainerLayout * pCL, UT_uint32 offset)
+{
+  m_bNeedsReformat = true;
+  if(getSectionLayout())
+    getSectionLayout()->setNeedsReformat(this);
+}
+
+void fl_EmbedLayout::updateLayout(bool bDoAll)
+{
+  fl_ContainerLayout * pBL = getFirstLayout();
+  while(pBL)
+  {
+    pBL->format();
+    pBL = pBL->getNext();
+  }
+}
 
 bool fl_EmbedLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 {
@@ -244,6 +265,10 @@ bool fl_EmbedLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 //
 // Remove all remaining structures
 //
+	if(getPrev())
+	{
+	  getPrev()->setNeedsReformat(getPrev());
+	}
 	collapse();
 //	UT_ASSERT(pcrx->getStruxType()== PTX_SectionFootnote);
 //
@@ -254,27 +279,9 @@ bool fl_EmbedLayout::doclistener_deleteStrux(const PX_ChangeRecord_Strux * pcrx)
 //
 // Fix the offsets for the block
 //
-	pEncBlock->updateOffsets(prevPos,0);
-
-	fl_ContainerLayout * pPrev = getPrev();
-	fl_ContainerLayout * pNext = getNext();
-
-	if(pPrev != NULL)
-	{
-		pPrev->setNext(pNext);
-	}
-	else
-	{
-		myContainingLayout()->setFirstLayout(pNext);
-	}
-	if(pNext != NULL)
-	{
-		pNext->setPrev(pPrev);
-	}
-	else
-	{
-		myContainingLayout()->setLastLayout(pPrev);
-	}
+	m_bHasEndFootnote = false;
+	pEncBlock->updateOffsets(prevPos,0,-getOldSize());
+	getSectionLayout()->remove(this);	
 	delete this;			// TODO whoa!  this construct is VERY dangerous.
 
 	return true;
@@ -337,6 +344,8 @@ fl_FootnoteLayout::~fl_FootnoteLayout()
 
 	setFirstContainer(NULL);
 	setLastContainer(NULL);
+
+	UT_return_if_fail( m_pLayout );
 	m_pLayout->removeFootnote(this);
 }
 
@@ -346,7 +355,7 @@ fl_FootnoteLayout::~fl_FootnoteLayout()
  */
 void fl_FootnoteLayout::_createFootnoteContainer(void)
 {
-	_lookupProperties();
+	lookupProperties();
 	fp_FootnoteContainer * pFootnoteContainer = new fp_FootnoteContainer(static_cast<fl_SectionLayout *>(this));
 	setFirstContainer(pFootnoteContainer);
 	setLastContainer(pFootnoteContainer);
@@ -367,7 +376,7 @@ void fl_FootnoteLayout::_createFootnoteContainer(void)
 
 /*!
   Create a new Footnote container.
-  \params If pPrevFootnote is non-null place the new cell after this in the linked
+  \param If pPrevFootnote is non-null place the new cell after this in the linked
           list, otherwise just append it to the end.
   \return The newly created Footnote container
 */
@@ -404,7 +413,7 @@ void fl_FootnoteLayout::_insertFootnoteContainer(fp_Container * pNewFC)
 			PT_DocPosition posBL = pBL->getPosition();
 			while(pRun && ((posBL + pRun->getBlockOffset() + pRun->getLength()) < posFL))
 			{
-				pRun = pRun->getNext();
+				pRun = pRun->getNextRun();
 			}
 			if(pRun && pRun->getLine())
 			{
@@ -476,15 +485,18 @@ void fl_FootnoteLayout::format(void)
 	m_bNeedsReformat = false;
 }
 
-void fl_FootnoteLayout::_lookupProperties(void)
-{
- 	const PP_AttrProp* pSectionAP = NULL;
 
-	m_pLayout->getDocument()->getAttrProp(m_apIndex, &pSectionAP);
+/*!
+    this function is only to be called by fl_ContainerLayout::lookupProperties()
+    all other code must call lookupProperties() instead
+*/
+void fl_FootnoteLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
+{
+	UT_return_if_fail(pSectionAP);
 	// I can't think of any properties we need for now.
 	// If we need any later, we'll add them. -PL
 	const XML_Char *pszFootnotePID = NULL;
-	if(!pSectionAP || !pSectionAP->getAttribute("footnote-id",pszFootnotePID))
+	if(!pSectionAP->getAttribute("footnote-id",pszFootnotePID))
 	{
 		m_iFootnotePID = 0;
 	}
@@ -563,6 +575,7 @@ fl_EndnoteLayout::fl_EndnoteLayout(FL_DocLayout* pLayout,
 					 PTX_SectionEndnote),
 	  m_iEndnotePID(0)
 {
+        UT_DEBUGMSG(("Create Endnote section %x from pos %d \n",this,getPosition()));
 	m_pLayout->addEndnote(this);
 	_createEndnoteContainer();
 }
@@ -596,7 +609,7 @@ fl_EndnoteLayout::~fl_EndnoteLayout()
  */
 void fl_EndnoteLayout::_createEndnoteContainer(void)
 {
-	_lookupProperties();
+	lookupProperties();
 	fp_EndnoteContainer * pEndnoteContainer = new fp_EndnoteContainer(static_cast<fl_SectionLayout *>(this));
 	setFirstContainer(pEndnoteContainer);
 	setLastContainer(pEndnoteContainer);
@@ -631,6 +644,7 @@ void fl_EndnoteLayout::_localCollapse(void)
 
 void fl_EndnoteLayout::collapse(void)
 {
+	UT_DEBUGMSG(("Collapsing  Endnote %x \n",this));
 	_localCollapse();
 	fp_EndnoteContainer *pFC = static_cast<fp_EndnoteContainer *>(getFirstContainer());
 	while(pFC)
@@ -699,15 +713,17 @@ void fl_EndnoteLayout::format(void)
 }
 
 
-void fl_EndnoteLayout::_lookupProperties(void)
+/*!
+    this function is only to be called by fl_ContainerLayout::lookupProperties()
+    all other code must call lookupProperties() instead
+*/
+void fl_EndnoteLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 {
- 	const PP_AttrProp* pSectionAP = NULL;
-
-	m_pLayout->getDocument()->getAttrProp(m_apIndex, &pSectionAP);
+	UT_return_if_fail(pSectionAP);
 	// I can't think of any properties we need for now.
 	// If we need any later, we'll add them. -PL
 	const XML_Char *pszEndnotePID = NULL;
-	if(!pSectionAP || !pSectionAP->getAttribute("endnote-id",pszEndnotePID))
+	if(!pSectionAP->getAttribute("endnote-id",pszEndnotePID))
 	{
 		m_iEndnotePID = 0;
 	}
@@ -720,7 +736,7 @@ void fl_EndnoteLayout::_lookupProperties(void)
 
 /*!
   Create a new Endote container.
-  \params If pPrevFootnote is non-null place the new cell after this in the linked
+  \param If pPrevFootnote is non-null place the new cell after this in the linked
           list, otherwise just append it to the end.
   \return The newly created Endnote container
 */
@@ -736,6 +752,7 @@ fp_Container* fl_EndnoteLayout::getNewContainer(fp_Container *)
 void fl_EndnoteLayout::_insertEndnoteContainer(fp_Container * pNewEC)
 {
 	UT_DEBUGMSG(("inserting endnote container into DocLayout list\n"));
+
 	m_pLayout->insertEndnoteContainer(static_cast<fp_EndnoteContainer *>(pNewEC));
 	m_bIsOnPage = true;
 }
