@@ -159,6 +159,18 @@ void 	fl_FrameLayout::setContainerProperties(void)
         {
 	        pFrame->setAbove(false);
 	}
+	else if(FL_FRAME_WRAPPED_TO_RIGHT == m_iFrameWrapMode)
+	{
+	        pFrame->setRightWrapped(true);
+	}
+	else if(FL_FRAME_WRAPPED_TO_LEFT == m_iFrameWrapMode)
+	{
+	  pFrame->setLeftWrapped(true);
+	}
+	else if(FL_FRAME_WRAPPED_TOPBOT == m_iFrameWrapMode)
+	{
+	        pFrame->setTopBot(true);
+	}
 //
 // Now do the image for this frame.
 //
@@ -616,6 +628,12 @@ void fl_FrameLayout::miniFormat(void)
 void fl_FrameLayout::format(void)
 {
 	xxx_UT_DEBUGMSG(("SEVIOR: Formatting first container is %x \n",getFirstContainer()));
+	if(isHidden() > FP_VISIBLE)
+	{
+		xxx_UT_DEBUGMSG(("Don't format FRAME coz I'm hidden! \n"));
+		return;
+	}
+
 	if(getFirstContainer() == NULL)
 	{
 		getNewContainer();
@@ -753,6 +771,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 
 	const XML_Char * pszBoundingSpace = NULL;
 	const XML_Char * pszTightWrapped = NULL;
+
 // Frame Type
 
 	if(!pSectionAP || !pSectionAP->getProperty("frame-type",pszFrameType))
@@ -826,6 +845,10 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 	{
 		m_iFrameWrapMode = FL_FRAME_WRAPPED_BOTH_SIDES;
 	}
+	else if(strcmp(pszWrapMode,"wrapped-topbot") == 0)
+	{
+		m_iFrameWrapMode = FL_FRAME_WRAPPED_TOPBOT;
+	}
 	else 
 	{
 		UT_DEBUGMSG(("Unknown wrap-mode %s \n",pszWrapMode));
