@@ -507,6 +507,8 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 						}
 					}
 				}
+				if(j< 0)
+				  j = 0;
 			}
 			pCol = static_cast<fp_Column *>(pCol->getFollower());
 		}
@@ -545,6 +547,10 @@ fp_Container * fp_Page::updatePageForWrapping(fp_Column *& pNextCol)
 	{
 		pNewFirstCon = static_cast<fp_Container *>(pNewFirstCon->getNext());
 	}
+	if(pNewFirstCon->getColumn() == NULL)
+	{
+	       return NULL;
+	}
 	pNextCol = static_cast<fp_Column *>(pNewFirstCon->getColumn());
 	pNewFirstCon = static_cast<fp_Container *>(pNextCol->getNthCon(0));
 	if(pNewFirstCon->getContainerType() == FP_CONTAINER_LINE)
@@ -807,9 +813,11 @@ void fp_Page::getScreenOffsets(fp_Container* pContainer, UT_sint32& xoff, UT_sin
 	    return;
 	}
 	m_pView->getPageScreenOffsets(this, xoff, yoff);
-
-	xoff += pContainer->getX();
-	yoff += pContainer->getY();
+	if(pContainer)
+        {
+	    xoff += pContainer->getX();
+	    yoff += pContainer->getY();
+	}
 }
 
 fp_Page* fp_Page::getNext(void) const
@@ -935,6 +943,7 @@ void fp_Page::draw(dg_DrawArgs* pDA, bool bAlwaysUseWhiteBackground)
 		{
 			pFC->setOverWrote();
 		}
+		
 		dg_DrawArgs da = *pDA;
 		if(m_pView && (m_pView->getViewMode() != VIEW_PRINT) && !pDA->pG->queryProperties(GR_Graphics::DGP_PAPER))
 		{
@@ -1924,7 +1933,7 @@ void fp_Page::mapXYToPosition(bool bNotFrames,UT_sint32 x, UT_sint32 y, PT_DocPo
 // change to show you can select the text box.
 //
 // If we're outside this distance make sure all other options are excluded
-// before placing hte point inside th text box
+// before placing the point inside the text box
 //
 
 				if(static_cast<UT_sint32>(iDist) > m_pLayout->getGraphics()->tlu(3))
@@ -2129,7 +2138,10 @@ void fp_Page::mapXYToPosition(bool bNotFrames,UT_sint32 x, UT_sint32 y, PT_DocPo
 
 	UT_ASSERT(pMinDist);
 
-	pMinDist->mapXYToPosition(x - pMinDist->getX(), y - pMinDist->getY(), pos, bBOL, bEOL,isTOC);
+	if (pMinDist)
+	{
+		pMinDist->mapXYToPosition(x - pMinDist->getX(), y - pMinDist->getY(), pos, bBOL, bEOL,isTOC);
+	}
 }
 
 void fp_Page::setView(FV_View* pView)
