@@ -86,6 +86,8 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 {
 	if(m_pDocument->isMarkRevisions())
 	{
+		
+		bool bHdrFtr = false;
 		// if the user selected the whole document for deletion, we will not delete the
 		// first block (we need always a visible block in any document); we make an
 		// exception to this in VDND, because in that case the original block is
@@ -178,8 +180,10 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 					case PTX_SectionTOC:
 						bHasEndStrux = true;
 						// fall through ...
-					case PTX_Section:
 					case PTX_SectionHdrFtr:
+						bHdrFtr = true;
+						// fall through
+					case PTX_Section:
 				    case PTX_EndFootnote:
 				    case PTX_EndEndnote:
 				    case PTX_EndFrame:
@@ -249,7 +253,7 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 				{
 					// if this fragment is one of the struxes that is paired with an end-strux, we
 					// need to delete both struxes and everything in between
-					if(bHasEndStrux)
+					if(bHasEndStrux || bHdrFtr)
 					{
 						PT_DocPosition posEnd = dposEnd;
 						for(pf_Frag * pf = pf1->getNext(); pf != NULL; pf = pf->getNext())
@@ -294,6 +298,10 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 									if(eStrux2Type != PTX_EndTOC)
 										continue;
 									break;
+								case PTX_SectionHdrFtr:
+									if(eStrux2Type != PTX_SectionHdrFtr)
+										continue;
+									break;
 							    default:
 									break;
 							}
@@ -305,11 +313,36 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 						
 						dposEnd = posEnd;
 					}
-					
-					if(!_realDeleteSpan(dpos1, dposEnd, p_AttrProp_Before,bDeleteTableStruxes,
-										bDontGlob))
-						return false;
 
+					if(bHdrFtr)
+					{
+						bHdrFtr = false; // only do this once
+						pf_Frag_Strux_SectionHdrFtr * pfHdr = static_cast<pf_Frag_Strux_SectionHdrFtr *>(pf1);
+
+						const PP_AttrProp * pAP = NULL;
+
+						if(!getAttrProp(pfHdr->getIndexAP(),&pAP) || !pAP)
+							return false;
+
+						const XML_Char * pszHdrId;
+						if(!pAP->getAttribute("id", pszHdrId) || !pszHdrId)
+							return false;
+
+						const XML_Char * pszHdrType;
+						if(!pAP->getAttribute("type", pszHdrType) || !pszHdrType)
+							return false;
+
+						// needs to be in this order because of undo
+						_realDeleteHdrFtrStrux(static_cast<pf_Frag_Strux*>(pf1));
+						_fixHdrFtrReferences(pszHdrType, pszHdrId);
+					}
+					else
+					{
+						if(!_realDeleteSpan(dpos1, dposEnd, p_AttrProp_Before,bDeleteTableStruxes,
+											bDontGlob))
+							return false;
+					}
+					
 					UT_uint32 iDeleteThisStep = dposEnd - dpos1;
 					
 					iRealDeleteCount += iDeleteThisStep;
@@ -348,6 +381,30 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 					if(! _realChangeStruxFmt(PTC_AddFmt, dpos1 + iLen, dpos1 + iLen /*2*iLen*/, ppRevAttrib,NULL,
 											 eStruxType,true))
 						return false;
+
+					if(bHdrFtr)
+					{
+						// even though this is just a notional removal, we still have to
+						// fix the references
+						bHdrFtr = false; // only do this once
+						pf_Frag_Strux_SectionHdrFtr * pfHdr = static_cast<pf_Frag_Strux_SectionHdrFtr *>(pf1);
+
+						const PP_AttrProp * pAP = NULL;
+
+						if(!getAttrProp(pfHdr->getIndexAP(),&pAP) || !pAP)
+							return false;
+
+						const XML_Char * pszHdrId;
+						if(!pAP->getAttribute("id", pszHdrId) || !pszHdrId)
+							return false;
+
+						const XML_Char * pszHdrType;
+						if(!pAP->getAttribute("type", pszHdrType) || !pszHdrType)
+							return false;
+
+						_fixHdrFtrReferences(pszHdrType, pszHdrId, true);
+					}
+					
 					break;
 
 				default:;
@@ -363,7 +420,117 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 							   bDontGlob);
 }
 
+/*!
+    scan piecetable and remove any references to the hdr/ftr located at pos dpos
+    bNotional indicates that the header has been marked deleted in revison mode, but not
+    physically removed from the document
+*/
+bool pt_PieceTable::_fixHdrFtrReferences(const XML_Char * pszHdrType, const XML_Char * pszHdrId,
+										 bool bNotional /* = false */)
+{
+	UT_return_val_if_fail( pszHdrType && pszHdrId, false );
+	
+	bool bRet = true;
+	const PP_AttrProp * pAP = NULL;
 
+	// look for any doc sections that referrence this header type and id
+	const pf_Frag * pFrag = m_fragments.getFirst();
+	while(pFrag)
+	{
+		if(pFrag->getType() == pf_Frag::PFT_Strux &&
+		   static_cast<const pf_Frag_Strux*>(pFrag)->getStruxType()==PTX_Section)
+		{
+			if(!getAttrProp(pFrag->getIndexAP(),&pAP) || !pAP)
+				continue;
+
+			// check for normal attribute
+			const XML_Char * pszMyHdrId = NULL;
+			if(pAP->getAttribute(pszHdrType, pszMyHdrId) && pszMyHdrId)
+			{
+				if(0 == strcmp(pszMyHdrId, pszHdrId))
+				{
+					const XML_Char* pAttrs [3];
+					pAttrs[0] = pszHdrType;
+					pAttrs[1] = "";
+					pAttrs[2] = NULL;
+
+					bRet &= _fmtChangeStruxWithNotify(PTC_SetFmt, (pf_Frag_Strux*)pFrag,
+													  pAttrs, NULL, false);
+				}
+			}
+
+			// now check for revision attribute ...
+			const XML_Char * pszRevision;
+			if(pAP->getAttribute("revision", pszRevision) && pszRevision)
+			{
+				bool bFound = false;
+				PP_RevisionAttr Revisions(pszRevision);
+
+				for(UT_uint32 i = 0; i < Revisions.getRevisionsCount(); ++i)
+				{
+					const PP_Revision * pRev = Revisions.getNthRevision(i);
+					UT_return_val_if_fail( pRev, false );
+
+					const XML_Char * pszMyHdrId = NULL;
+					if(pRev->getAttribute(pszHdrType, pszMyHdrId) && pszMyHdrId)
+					{
+						if(0 != strcmp(pszHdrId, pszMyHdrId))
+							continue;
+
+						if(!bNotional)
+						{
+							// NB: this is safe, since we own the PP_RevisionAttr object
+							// of local scope which in turn owns this revisions
+							const_cast<PP_Revision*>(pRev)->setAttribute(pszHdrType, "");
+						}
+						else
+						{
+							UT_uint32 iId = m_pDocument->getRevisionId();
+							UT_uint32 iMinId;
+							const PP_Revision * pRev = Revisions.getRevisionWithId(iId, iMinId);
+							if(pRev)
+							{
+								// NB: this is safe, since we own the PP_RevisionAttr object
+								// of local scope which in turn owns this revisions
+								const_cast<PP_Revision*>(pRev)->setAttribute(pszHdrType, "");
+							}
+							else
+							{
+								// we have a section that references this header in
+								// previous revision and has no changes in the current
+								// revision, so we need to add a new revisions in which
+								// the header is not referenced
+								const XML_Char * pAttrs [3] = {pszHdrType, pszHdrId, NULL};
+								Revisions.addRevision(iId, PP_REVISION_FMT_CHANGE, pAttrs, NULL);
+							}
+						}
+
+						Revisions.forceDirty();
+						bFound = true;
+					}
+				}
+				
+				if(bFound)
+				{
+					const XML_Char* pAttrs [3];
+					pAttrs[0] = "revision";
+					pAttrs[1] = Revisions.getXMLstring();
+					pAttrs[2] = NULL;
+
+					bRet &= _fmtChangeStruxWithNotify(PTC_SetFmt, (pf_Frag_Strux*)pFrag,
+													  pAttrs, NULL, false);
+				}
+			}
+								
+								
+		}
+							
+		pFrag = pFrag->getNext();
+	}
+
+
+	return bRet;
+}
 
 bool pt_PieceTable::_deleteSpan(pf_Frag_Text * pft, UT_uint32 fragOffset,
 								PT_BufIndex bi, UT_uint32 length,
@@ -1134,9 +1301,20 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 // 
 					if(!isFootnote(pfs) && !isEndFootnote(pfs))
 					{ 
-						UT_DEBUGMSG(("Delete Block strux \n"));
+						UT_DEBUGMSG(("Delete Block strux dpos1 %d \n",dpos1));
 						bResult = _deleteStruxWithNotify(dpos1,pfs,
-												  &pfNewEnd,&fragOffsetNewEnd);
+										 &pfNewEnd,&fragOffsetNewEnd);
+						
+						if(!bResult) // can't delete this block strux
+						             // but can delete the rest of the content
+						{
+						  UT_DEBUGMSG(("dpos1 = %d \n",dpos1));
+						  pfNewEnd = pfs->getNext();
+						  dpos1 += pfs->getLength();
+						  pfsContainer = pfs;
+						  fragOffsetNewEnd = 0;
+						}
+						// UT_return_val_if_fail(bResult,false);
 						bPrevWasCell = false;
 						bPrevWasFrame = false;
 						bPrevWasEndTable = false;
@@ -1397,7 +1575,7 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
 			UT_return_val_if_fail (bResult, false);
 		}
 		break;
-		// the bookmark and hyperlink objects require special treatment; since
+		// the bookmark and hyperlink objects require specxial treatment; since
 		// they come in pairs, we have to ensure that both are deleted together
 		// so we have to find the other part of the pair, delete it, adjust the
 		// positional variables and the delete the one we were originally asked
