@@ -54,7 +54,9 @@ pt_PieceTable::pt_PieceTable(PD_Document * pDocument)
 	m_pDocument(pDocument),
     m_atomicGlobCount(0),
 	m_bDoingTheDo(false),
-	m_bDoNotTweakPosition(false)
+	m_bDoNotTweakPosition(false),
+	m_iXID(0),
+	m_iCurCRNumber(0)
 {
 
 	setPieceTableState(PTS_Create);
@@ -558,7 +560,57 @@ PT_DocPosition pt_PieceTable::getStruxPosition(PL_StruxDocHandle sdh) const
 
 void pt_PieceTable::deleteHdrFtrStrux(pf_Frag_Strux * pfs)
 {
-	_realDeleteHdrFtrStrux(pfs);
+	UT_return_if_fail( pfs );
+	
+	if(m_pDocument->isMarkRevisions())
+	{
+		const pf_Frag * pfFrag = NULL;
+		pfFrag = static_cast<const pf_Frag *>(pfs);
+		PT_DocPosition dpos1 = getFragPosition(pfFrag);
+
+		pfFrag = pfFrag->getNext();
+
+		while(pfFrag)
+		{
+			if((pfFrag->getType() == pf_Frag::PFT_EndOfDoc) ||
+			   (pfFrag->getType() == pf_Frag::PFT_Strux &&
+				static_cast<const pf_Frag_Strux*>(pfFrag)->getStruxType() == PTX_SectionHdrFtr))
+			{
+				break;
+			}
+
+			pfFrag = pfFrag->getNext();
+		}
+
+		// there should always be at least EndOfDoc fragment !!!
+		UT_return_if_fail( pfFrag );
+		
+		PT_DocPosition dpos2 = getFragPosition(pfFrag);
+
+		UT_uint32 iRealDelete = 0;
+		deleteSpan(dpos1, dpos2, NULL, iRealDelete, true /*delete table struxes*/, false /* don't glob */);
+	}
+	else
+	{
+		const PP_AttrProp * pAP = NULL;
+		UT_return_if_fail(pfs->getStruxType()==PTX_SectionHdrFtr);
+		pf_Frag_Strux_SectionHdrFtr * pfHdr = static_cast<pf_Frag_Strux_SectionHdrFtr *>(pfs);
+
+		if(!getAttrProp(pfHdr->getIndexAP(),&pAP) || !pAP)
+			return;
+
+		const XML_Char * pszHdrId;
+		if(!pAP->getAttribute("id", pszHdrId) || !pszHdrId)
+			return;
+
+		const XML_Char * pszHdrType;
+		if(!pAP->getAttribute("type", pszHdrType) || !pszHdrType)
+			return;
+
+		_realDeleteHdrFtrStrux(pfs);
+		_fixHdrFtrReferences(pszHdrType, pszHdrId);
+	}
+	
 }
 
 void pt_PieceTable::_realDeleteHdrFtrStrux(pf_Frag_Strux * pfs)
@@ -1129,7 +1181,7 @@ bool pt_PieceTable::_changePointWithNotify(PT_DocPosition dpos)
 {
 	PX_ChangeRecord * pcr
 		= new PX_ChangeRecord(PX_ChangeRecord::PXT_ChangePoint,
-							  dpos, 0);
+							  dpos, 0,0);
 	UT_return_val_if_fail (pcr,false);
 
 	m_history.addChangeRecord(pcr);
@@ -1138,5 +1190,26 @@ bool pt_PieceTable::_changePointWithNotify(PT_DocPosition dpos)
 	return true;
 }
 
+/*!
+    This function crawls the entire PT and assignes new xid to any fragment that should
+    have one and does not. It is primarily to be used by exporters (accessed throught
+    PD_Document wrapper)
+*/
+void pt_PieceTable::fixMissingXIDs()
+{
+	for (pf_Frag * pf = m_fragments.getFirst(); (pf); pf=pf->getNext())
+	{
+		if(!pf->getXID() && pf->usesXID())
+			pf->setXID(getXID());
+	}
+}
+
+UT_uint32 pt_PieceTable::getXID()
+{
+	++m_iXID;
+	return m_iXID;
+}
+
+
 
 
